#!/usr/bin/env bash

# This script is called by p1.test.user.bash
# It runs as atlascdr on the test host.

if [ x"$1" == x ]; then
  echo "usage: $0 {CastorScript config file}"
  exit 1
fi

cd `dirname $0`
CS_CONFIG_FILE="$1"

SOURCE_DIR=`pwd`/..
#REMOTE_DIR="/eos/atlas/atlastier0/daq/test/CastorScript.tests"
REMOTE_DIR=$(sed -rn "s/^CopyDir: '(.*)'/\1/p" $CS_CONFIG_FILE)
BASEFILENAME="dummy"
EOSSTAGEHOST=$(sed -rn "s/^StageHost: '(.*)'/\1/p" $CS_CONFIG_FILE)

echo "Cleaning remote EOS test area: $REMOTE_DIR"
KEYTAB_FILE="/daq_area/castor/`hostname -s`/atlascdr/atlascdr.keytab"
kinit -kt $KEYTAB_FILE atlascdr
for i in `seq 0 9`; do
	echo "Removing: ${BASEFILENAME}$i.data"
	xrdfs $EOSSTAGEHOST rm $REMOTE_DIR/${BASEFILENAME}$i.data &> /dev/null
done

TMP_DIR=/tmp/atlascdr
echo "Cleaning tmp directory: $TMP_DIR"
rm -rf $TMP_DIR
mkdir -p $TMP_DIR/logs

echo "Creating dummy data files"
for i in `seq 0 9`; do
	dd if=/dev/zero of=$TMP_DIR/${BASEFILENAME}$i.data bs=1k count=1
done

echo "Starting CastorScript with config file: $CS_CONFIG_FILE"
source $SOURCE_DIR/script_setup.sh
python -u $SOURCE_DIR/Script/CastorScript.py $CS_CONFIG_FILE &>$TMP_DIR/logs/log.out &

# through ssh and su, TERM is not set and tput is lost
export TERM=xterm

echo "Watching files to transfer"
NF=`find $TMP_DIR -maxdepth 1 -name ${BASEFILENAME}* | wc -l`
for i in `seq 0 $NF`; do echo; done
for i in `seq 0 $NF`; do tput cuu1; done

while [ "0$NF" -gt 0 ] ; do
        LS_OUTPUT=`ls -l $TMP_DIR/${BASEFILENAME}*`
        NL=`echo "$LS_OUTPUT" | wc -l`
        echo "$LS_OUTPUT"

        sleep 1

        for i in `seq 0 $NL`; do tput cuu1; done
        for i in `seq 0 $NL`; do
          printf ' %.0s' {1..160}; printf '\n'
        done
        for i in `seq 0 $((NL-1))`; do tput cuu1; done

        NF=`find $TMP_DIR -maxdepth 1 -name ${BASEFILENAME}* | wc -l`
done

echo "Stopping CastorScript"
~flegoff/castor_tools/signal-script.sh 12

echo "Cleaning remote files"
for i in $(seq 0 9); do
	echo "Removing: $REMOTE_DIR/${BASEFILENAME}$i.data"
	xrdfs $EOSSTAGEHOST rm $REMOTE_DIR/${BASEFILENAME}$i.data
done

NB_ERR=$(grep -c -e WARNING -e ERROR -r $TMP_DIR/logs | awk -F: 'BEGIN{sum=0} //{sum += $2} END{print sum}')
echo "number of errors and warnings in the log files: $NB_ERR"

if [ $NB_ERR -eq 0 ]; then
	echo "deleting tmp dir (including logs)"
	rm -rf $TMP_DIR
else
	echo "Check logs at: $TMP_DIR/logs"
	echo "Don't forget to: rm -rf $TMP_DIR"
fi
