from future import standard_library
standard_library.install_aliases()
import unittest
import time
import threading
import os
import logging
import sys

if __name__ == '__main__':
    SCRIPT_DIR = os.path.abspath(os.path.join(os.path.dirname(__file__), '../../Script'))
    sys.path.append(SCRIPT_DIR)

from cs.Threads.ERSThread import ERSThread
import cs.Tools.ConfigParser as ConfigParser

class TestERSThread(unittest.TestCase):
    base_file_name = 'ers_thread_test'

    def instantiate_handler_and_log_all_severities(self):
        base_dir = os.path.abspath(os.path.join(os.path.dirname(__file__), '../..'))
        config = ConfigParser.ConfigHolder(
                os.path.join(base_dir, 'Configs/template.cfg'))

        # override some config values
        config.ERSTimeout = 1
        config.LogDir = '.'
        # redirect ers to files (no idea why it does not work when using
        config.ers_error = 'file({}.error)'.format(TestERSThread.base_file_name)
        # the same file for all severity)
        config.ers_warning = 'file({}.warning)'.format(TestERSThread.base_file_name)
        config.ers_info = 'file({}.info)'.format(TestERSThread.base_file_name)
        config.ers_debug = 'file({}.debug)'.format(TestERSThread.base_file_name)

        event = threading.Event()
        ers_thread = ERSThread(config, event)
        ers_thread.start()
        time.sleep(1)

        self.assertIsNotNone(ers_thread.ers_handler)

        # allow all log levels at the root of loggers
        logging.getLogger().setLevel(logging.DEBUG)
        logging.error("error")
        logging.warning("warning")
        logging.info("info")
        logging.debug("debug")

        # stop thread
        ers_thread.ersExit()
        ers_thread.join()

        for ext in ['error', 'warning', 'info', 'debug']:
            with open(TestERSThread.base_file_name + '.' + ext, 'r') as ers_log_file:
                lines = ers_log_file.readlines()
                self.assertGreaterEqual(len(lines), 1)
                self.assertTrue(ext in lines[0])


    @classmethod
    def setUpClass(cls):
        TestERSThread.tearDownClass()


    @classmethod
    def tearDownClass(cls):
        try:
            os.remove('ERSLog.out')
            for ext in ['error', 'warning', 'info', 'debug']:
                os.remove(TestERSThread.base_file_name + '.' + ext)
        except OSError:
            # ignore missing files
            pass


if __name__ == '__main__':
    unittest.main()
