from builtins import str
import unittest
import signal
import shutil
from subprocess import Popen

if __name__ == '__main__':
    import sys
    from os.path import dirname, abspath, join
    # add CastorScript/Script to path so imports keeps working
    SCRIPT_DIR = abspath(join(dirname(__file__), '../..'))
    sys.path.append(SCRIPT_DIR)

from UnitTests.FunctionalTests.FuncTestUtils.DirUtils import DirectoryChecker

from UnitTests.FunctionalTests.FuncTestUtils.DirUtils import createTmpDir


# I'm a developer, I want to see if the script can copy files.
class TestCanCopyFilesLocally(unittest.TestCase):

    @classmethod
    def setUpClass(self):

        # I create test directories and remember their locations
        ## returns a dictionary over dirs like src des log config etc..
        self.dirs = createTmpDir("/Script/UnitTests/FunctionalTests")

        ## Create test file
        test_file = open('{}/file.data'.format(self.dirs["src"]), "w")
        test_file.close()

        # I make a config file from the template (CastorScript/Configs)
        # I find the config file
        # I open it
        cfgTemplate = open(self.dirs["config"] + "/template.cfg", "r")

        # I make a new file in CastorScript/tmp directory
        cfgTest = open(self.dirs["tmp"] + "/funcTestConf.cfg", "w")

        # And I copy the content and replaces the directories with my own dirs        
        custom_cfg_lines = {
            "LogDir: ": "LogDir: \'{}\'\n".format(self.dirs["log"]),
            "DirList: ": "DirList: [\'{}\',]\n".format(self.dirs["src"]),
            "CopyDir: ": "CopyDir: \'{}\'\n".format(self.dirs["des"]),
                }

        # And I copy the content and replaces the directories with my own dirs        
        for line in cfgTemplate:
            for key, value in custom_cfg_lines.items():
                if line.find(key) is not -1:
                    line = value
            cfgTest.write(line)
        cfgTest.close()

        ## Run the script with config
        scriptcommand = ["python", self.dirs["project"] + "/Script/CastorScript.py", self.dirs["tmp"] + "/funcTestConf.cfg" ]
        self.castorscriptprocess = Popen(scriptcommand)


    @classmethod
    def tearDownClass(self):
        # stop process
        self.castorscriptprocess.send_signal(signal.SIGUSR2)
        self.castorscriptprocess.wait()
        # remove directories and files
        shutil.rmtree(self.dirs["tmp"])


    # Test cases:

    def test_localCopy_script_has_been_instanciated(self):
        #test if the script runs by Instance
        self.assertNotEqual(self.castorscriptprocess, None, msg="Castorscript, is not running; Subprocess is of type none")


    def test_localCopy_can_produce_logfiles(self):
        
        seconds = 10
        timeout, output = DirectoryChecker(self.dirs["log"]).waitForAnyFile(seconds)
        
        timeoutMessage = "Timeout, found no log files within a timeframe of " + str(seconds) + " seconds"
        self.assertFalse(timeout, msg=timeoutMessage)

        outNotFoundMessage = "No files ending with .out found in log directory, found:" + output
        self.assertTrue(output.find(".out") > -1, msg=outNotFoundMessage)


    def test_localCopy_can_copy_files_to_des_directory(self):
        
        seconds = 4
        timeout, output = DirectoryChecker(self.dirs["des"]).waitForAnyFile(seconds)
        
        timeoutMessage = "Timeout, found no files within a timeframe of " + str(seconds) + " seconds"
        self.assertFalse(timeout, msg=timeoutMessage)

        dataNotFoundMessage = "No files ending with .data found in des directory, found:" + output
        self.assertTrue(output.find(".data") > -1, msg=dataNotFoundMessage)


    def test_localCopy_src_files_was_deleted(self):

        seconds = 5
        timeout, output = DirectoryChecker(self.dirs["src"]).waitForEmptyDir(seconds)

        timeoutMessage = "Timeout, found no empty dir within a timeframe of " + str(seconds) + " seconds"
        self.assertFalse(timeout, msg=timeoutMessage)

        logsNotFoundMessage = "No files ending with .out found in log directory, found:" + output
        self.assertTrue(output.find("file") == -1, msg=logsNotFoundMessage)


if __name__ == '__main__':
    unittest.main()
