from future import standard_library
standard_library.install_aliases()
import unittest
import mock
import threading
import queue

if __name__ == '__main__':
    import sys
    from os.path import dirname, abspath, join
    ## add CastorScript/Script to path so imports keeps working
    SCRIPT_DIR = abspath(join(dirname(__file__), '..'))
    sys.path.append(SCRIPT_DIR)

from cs.Threads.InfoServiceThread import InfoServiceThread
from cs.Threads.CopyThread import CopyThread
from cs.Threads.DeleteThread import DeleteThread
from cs.Threads.ManagerThread import ManagerThread
from cs.Tools.ConfigParser import ConfigHolder

# as a developer I want to publish information to the information service
class TestInfoServiceThread(unittest.TestCase):
    @classmethod
    def setUpClass(self):
        ## we mock the event so that the thread won't be left hanging
        self.mock_event = mock.create_autospec(threading.Event())
        kwargs = {"wait.return_value": True}
        self.mock_event.configure_mock(**kwargs)

        ## we mock the copythread with queues
        self.mock_CopyThread = mock.create_autospec(CopyThread)

        self.copy_queue = queue.Queue(3)
        self.copy_queue.put("item 1")
        self.copy_queue.put("item 2")

        self.delete_queue = queue.Queue(3)
        self.delete_queue.put("item 3")
        self.delete_queue.put("item 4")
        self.delete_queue.put("item 5")

        self.mock_CopyThread.configure_mock(
            CopyQueue = self.copy_queue,
            DeleteQueue = self.delete_queue,
            simple_transfer_failures = 5,
            problematic_transfer_failures = 5,
            files_copied = 5,
            CopyList = ["one","two","three"],
            copy_lock = threading.Lock())
        kwargs_copy = {"getCopyListLength.return_value": 3}
        self.mock_CopyThread.configure_mock(**kwargs_copy)


        ## we mock the deletethread
        self.mock_DeleteThread = mock.create_autospec(DeleteThread)
        self.mock_DeleteThread.configure_mock(files_deleted=5)

        ## we mock the managerthread
        self.mock_ManagerThread = mock.create_autospec(ManagerThread)
        self.mock_ManagerThread.configure_mock(ProblDict={"one":1, "two":2},
            manager_lock = threading.Lock())

        ## we mock the configuration
        self.mock_cfg = mock.create_autospec(ConfigHolder)
        self.mock_cfg.configure_mock(ERSPartition="initial",
            LogDir="",
            LogLevel="",
            FileName="Conf_test.cfg",
            IS_partition="initial",
            IS_server="RunParams",
            IS_publication_period="1",
            fs_info_publishing_enabled=True,
            IS_wait_after_partition_valid=60)

        ## we're not interested in any logging for this test
        with mock.patch("cs.Threads.InfoServiceThread.enable_file_logging"):
            self.info_thread = InfoServiceThread(self.mock_cfg, self.mock_event, self.mock_CopyThread, self.mock_DeleteThread, self.mock_ManagerThread)

class TestInfoServiceExit(TestInfoServiceThread):
    def test_it_should_stop_thread_execution(self):
        # I run the thread
        self.info_thread.start()
        # and shut it down
        self.info_thread.info_service_exit()
        self.info_thread.join(3.0)

        # and I see that the thread has been killed
        self.assertFalse(self.info_thread.is_alive(), msg="InfoService Thread didn't die")


class TestUpdateCSInfo(TestInfoServiceThread):

    def test_it_should_populate_the_IS_data_object(self):
        mock_ISObject = mock.Mock()
        with mock.patch('ispy.ISObject', return_value=mock_ISObject):
            self.info_thread.update_cs_info()

            self.assertGreater(mock_ISObject.uptime_seconds, 0.0)
            self.assertEqual(mock_ISObject.files_to_be_copied, 2)
            self.assertEqual(mock_ISObject.files_to_be_deleted, 3)
            self.assertEqual(mock_ISObject.files_being_copied, 3)
            self.assertEqual(mock_ISObject.files_copied,5)
            self.assertEqual(mock_ISObject.files_deleted,5)
            self.assertEqual(mock_ISObject.simple_transfer_failures,5)
            self.assertEqual(mock_ISObject.problematic_transfer_failures,5)
            self.assertEqual(mock_ISObject.files_currently_problematic,2)



class TestGetProcessUptimeInSeconds(TestInfoServiceThread):
    def test_it_should_return_a_positive_value(self):
        self.assertGreaterEqual(self.info_thread.get_process_uptime_in_seconds(), 0)

#TODO: WRITE TEST for catching following exception
#ALSO WRITE PARAMETER IN CONFIG FILE
## ERROR WHEN NEED TO MIGRATE TO NEW FORMAT
# 18:08:47 ERROR CastorScript-pc-tbed-pub-28.cern.ch-tbedconfigISERSeosstorage Message Error occured in send_update(): <class 'is.InfoNotCompatible'>
#2019-May-03 14:19:50,948 ERROR [send_update at InfoServiceThread.py:88] Error occured in send_update(): <class 'is.InfoNotCompatible'>
#The type of the IS infomation 'RunParams.CastorScript-pc-tbed-pub-28.cern.ch-tbedconfigISERSeosstorage.State' is not compatible with the one of the given object

## ERROR WHEN PUBLISHING WRONG DATA
#2019-May-03 14:09:42,662 ERROR [send_update at InfoServiceThread.py:88] Error occured in send_update(): <class 'is.AttributeNotFound'>
#Can not find the 'fakenews' attribute in the IS meta-data description


if __name__ == "__main__":
    unittest.main()
