#!/bin/bash
SHORTHOSTNAME=`hostname -s`

#edit these two when testing.
SWDIR="/sw/castor"
CONFDIR="/daq_area/castor/${SHORTHOSTNAME}/${USER}"

# Prevents "*.cfg" to be returned if no files are present
shopt -s nullglob

for CFG in $CONFDIR/*.cfg; do

	#get logdir from config file (find LogDir: in CFG | pipe 2nd part | del all ' & " | del last char if == / )
	logdir=$(grep 'LogDir:' $CFG | awk '{print $2}' | tr -d "'" | tr -d "\"" | sed "s/\/$//")
	#get is parameters from config file
	IS_PARTITION_NAME=$(grep 'IS_partition' $CFG | awk '{print $2}' | tr -d "'" | tr -d "\"")
	IS_SERVER=$(grep 'IS_server: ' $CFG | awk '{print $2}' | tr -d "'" | tr -d "\"")

	#check if a python process running the castorscript, with this config is running
	ps -ef | grep python | grep CastorScript.py | grep "$CFG" > /dev/null 2>&1

	if [ $? -eq 1 ]; then

		# If the CastorScript crashed, especially on uncaught exceptions, this watchdog
		# restarts it and we won't notice. Uncaught exception messages end up in
		# log.out: mail it to Fabrice if the file has non-zero length.
		# But first filter out unavoidable ERS messages which are not interesting for us
		# These messages appear when the partition used for ERS logging exits, before
		# the ERS thread has the time to remove the ERS handler.
		cp $logdir/log.out $logdir/log.out.raw
		sed '/Partition ".*" does not exist/d;/CORBA system exception "TRANSIENT(1096024066=TRANSIENT_ConnectFailed)" has been raised/d' -i $logdir/log.out
		if [ -s $logdir/log.out ]; then
			echo "$CFG" | mail -s "CastorScript instance crashed" -a $logdir/log.out -r "atlascdr@cern.ch" flegoff@cern.ch
		fi

		# Backup previous running instance log files
		olddir=logbackup_$(date "+%Y%m%d-%H%M%S")
		mkdir -p $logdir/$olddir
		mv $logdir/*.out $logdir/$olddir
		find $logdir -maxdepth 1 -name logbackup\* -printf "%T@ %p\n" | sort -n | head -n -5 | awk '{print $2}' | xargs rm -rf

		# Source required environment
		source $SWDIR/script_setup.sh

		#use castorscript functions to generate is_info_name for consistency
		IS_INFO_NAME=$(python $SWDIR/print_is_app_name.py $CFG $IS_SERVER)

		#write to IS that this instance has an uptime of 0.0 to indicate it is offline, but leave other parameters as is (read before write)
		is_write -p $IS_PARTITION_NAME -n $IS_INFO_NAME -t CastorScriptState -r -a uptime_seconds -v 0.0 > /dev/null 2>&1

		# Start script
		python -u $SWDIR/Script/CastorScript.py $CFG > $logdir/log.out 2>&1 &

		# Log messages
		date
		echo "Copying script restarted: $CFG"
		echo
	fi

done
